/* ------------------------------------------------------------------------- *
 * Copyright 2002-2023, OpenNebula Project, OpenNebula Systems               *
 *                                                                           *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may   *
 * not use this file except in compliance with the License. You may obtain   *
 * a copy of the License at                                                  *
 *                                                                           *
 * http://www.apache.org/licenses/LICENSE-2.0                                *
 *                                                                           *
 * Unless required by applicable law or agreed to in writing, software       *
 * distributed under the License is distributed on an "AS IS" BASIS,         *
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  *
 * See the License for the specific language governing permissions and       *
 * limitations under the License.                                            *
 * ------------------------------------------------------------------------- */
import { useEffect, useMemo, ReactElement } from 'react'
import PropTypes from 'prop-types'

import { SortDown, ArrowDown, ArrowUp } from 'iconoir-react'
import { MenuItem, MenuList, Chip, Stack } from '@mui/material'
import {
  TableInstance,
  UseSortByInstanceProps,
  UseSortByState,
} from 'react-table'

import HeaderPopover from 'client/components/Header/Popover'
import { Translate } from 'client/components/HOC'
import { T } from 'client/constants'

/**
 * Render all selected sorters.
 *
 * @param {object} props - Props
 * @param {string} [props.className] - Class name for the container
 * @param {TableInstance} props.useTableProps - Table props
 * @returns {ReactElement} Component JSX
 */
const GlobalSort = ({ className, useTableProps }) => {
  const { headers, state } = useTableProps

  /** @type {UseSortByInstanceProps} */
  const { setSortBy } = useTableProps

  /** @type {UseSortByState} */
  const { sortBy } = state

  const headersNotSorted = useMemo(
    () =>
      headers.filter(
        ({ isSorted, canSort, isVisible }) => !isSorted && canSort && isVisible
      ),
    [sortBy.length]
  )

  const handleClick = (id, name) => {
    setSortBy([{ id, desc: false, name }, ...sortBy])
  }

  const handleDelete = (removeId) => {
    setSortBy(sortBy.filter(({ id }) => id !== removeId))
  }

  const handleToggle = (id, desc) => {
    setSortBy(sortBy.map((sort) => (sort.id === id ? { ...sort, desc } : sort)))
  }

  useEffect(() => () => setSortBy([]), [])

  return !headersNotSorted.length && !sortBy.length ? null : (
    <Stack className={className} direction="row" gap="0.5em" flexWrap="wrap">
      {useMemo(
        () => (
          <HeaderPopover
            id="sort-by-button"
            icon={<SortDown />}
            buttonLabel={T.SortBy}
            buttonProps={{
              'data-cy': 'sort-by-button',
              disabled: headersNotSorted.length === 0,
              variant: 'outlined',
              color: 'secondary',
            }}
            popperProps={{ placement: 'bottom-end' }}
          >
            {() => (
              <MenuList>
                {headersNotSorted?.map(({ id, Header: name }) => (
                  <MenuItem key={id} onClick={() => handleClick(id, name)}>
                    <Translate word={name} />
                  </MenuItem>
                ))}
              </MenuList>
            )}
          </HeaderPopover>
        ),
        [headersNotSorted.length]
      )}

      {useMemo(
        () =>
          sortBy?.map(({ name, id, desc }) => (
            <Chip
              key={`${id}-${desc ? 'desc' : 'asc'}`}
              icon={desc ? <ArrowUp /> : <ArrowDown />}
              label={name ?? id}
              onClick={() => handleToggle(id, !desc)}
              onDelete={() => handleDelete(id)}
            />
          )),
        [sortBy.length, handleToggle]
      )}
    </Stack>
  )
}

GlobalSort.propTypes = {
  className: PropTypes.string,
  useTableProps: PropTypes.object.isRequired,
}

export default GlobalSort
