/* ------------------------------------------------------------------------- *
 * Copyright 2002-2023, OpenNebula Project, OpenNebula Systems               *
 *                                                                           *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may   *
 * not use this file except in compliance with the License. You may obtain   *
 * a copy of the License at                                                  *
 *                                                                           *
 * http://www.apache.org/licenses/LICENSE-2.0                                *
 *                                                                           *
 * Unless required by applicable law or agreed to in writing, software       *
 * distributed under the License is distributed on an "AS IS" BASIS,         *
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  *
 * See the License for the specific language governing permissions and       *
 * limitations under the License.                                            *
 * ------------------------------------------------------------------------- */
/* eslint-disable jsdoc/require-jsdoc */
import { useMemo } from 'react'
import { useHistory } from 'react-router-dom'
import {
  AddSquare,
  Import,
  Trash,
  PlayOutline,
  Lock,
  Group,
  Cart,
} from 'iconoir-react'

import { useViews } from 'client/features/Auth'
import {
  useLockTemplateMutation,
  useUnlockTemplateMutation,
  useCloneTemplateMutation,
  useRemoveTemplateMutation,
} from 'client/features/OneApi/vmTemplate'
import { Tr, Translate } from 'client/components/HOC'

import { CloneForm } from 'client/components/Forms/VmTemplate'
import { createActions } from 'client/components/Tables/Enhanced/Utils'
import { PATH } from 'client/apps/sunstone/routesOne'

import { T, VM_TEMPLATE_ACTIONS, RESOURCE_NAMES } from 'client/constants'

const MessageToConfirmAction = (rows) => {
  const names = rows?.map?.(({ original }) => original?.NAME)

  return (
    <>
      <p>
        <Translate word={T.VMTemplates} />
        {`: ${names.join(', ')}`}
      </p>
      <p>
        <Translate word={T.DoYouWantProceed} />
      </p>
    </>
  )
}

MessageToConfirmAction.displayName = 'MessageToConfirmAction'

const Actions = () => {
  const history = useHistory()
  const { view, getResourceView } = useViews()
  const [lock] = useLockTemplateMutation()
  const [unlock] = useUnlockTemplateMutation()
  const [clone] = useCloneTemplateMutation()
  const [remove] = useRemoveTemplateMutation()

  return useMemo(
    () =>
      createActions({
        filters: getResourceView(RESOURCE_NAMES.VM_TEMPLATE)?.actions,
        actions: [
          {
            accessor: VM_TEMPLATE_ACTIONS.CREATE_DIALOG,
            tooltip: T.Create,
            icon: AddSquare,
            action: () => history.push(PATH.TEMPLATE.VMS.CREATE),
          },
          {
            accessor: VM_TEMPLATE_ACTIONS.IMPORT_DIALOG,
            tooltip: T.Import,
            icon: Import,
            selected: { max: 1 },
            disabled: true,
            action: (rows) => {
              // TODO: go to IMPORT form
            },
          },
          {
            accessor: VM_TEMPLATE_ACTIONS.INSTANTIATE_DIALOG,
            tooltip: T.Instantiate,
            icon: PlayOutline,
            selected: { max: 1 },
            action: (rows) => {
              const template = rows?.[0]?.original ?? {}
              const path = PATH.TEMPLATE.VMS.INSTANTIATE

              history.push(path, template)
            },
          },
          {
            accessor: VM_TEMPLATE_ACTIONS.CREATE_APP_DIALOG,
            tooltip: T.CreateMarketApp,
            selected: { max: 1 },
            icon: Cart,
            action: (rows) => {
              const template = rows?.[0]?.original ?? {}
              const path = PATH.STORAGE.MARKETPLACE_APPS.CREATE

              history.push(path, [RESOURCE_NAMES.VM_TEMPLATE, template])
            },
          },
          {
            accessor: VM_TEMPLATE_ACTIONS.UPDATE_DIALOG,
            label: T.Update,
            tooltip: T.Update,
            selected: { max: 1 },
            color: 'secondary',
            action: (rows) => {
              const vmTemplate = rows?.[0]?.original ?? {}
              const path = PATH.TEMPLATE.VMS.CREATE

              history.push(path, vmTemplate)
            },
          },
          {
            accessor: VM_TEMPLATE_ACTIONS.CLONE,
            label: T.Clone,
            tooltip: T.Clone,
            selected: true,
            color: 'secondary',
            options: [
              {
                dialogProps: {
                  title: (rows) => {
                    const isMultiple = rows?.length > 1
                    const { ID, NAME } = rows?.[0]?.original ?? {}

                    return [
                      Tr(
                        isMultiple ? T.CloneSeveralTemplates : T.CloneTemplate
                      ),
                      !isMultiple && `#${ID} ${NAME}`,
                    ]
                      .filter(Boolean)
                      .join(' - ')
                  },
                },
                form: (rows) => {
                  const vmTemplates = rows?.map(({ original }) => original)
                  const stepProps = { isMultiple: vmTemplates.length > 1 }
                  const initialValues = {
                    name: `Copy of ${vmTemplates?.[0]?.NAME}`,
                  }

                  return CloneForm({ stepProps, initialValues })
                },
                onSubmit: (rows) => async (formData) => {
                  const { prefix, ...restOfData } = formData

                  const vmTemplates = rows?.map?.(
                    ({ original: { ID, NAME } = {} }) => {
                      // overwrite all names with prefix+NAME
                      const name = prefix ? `${prefix} ${NAME}` : NAME

                      return { id: ID, ...restOfData, name }
                    }
                  )

                  await Promise.all(vmTemplates.map(clone))
                },
              },
            ],
          },
          {
            tooltip: T.Ownership,
            icon: Group,
            selected: true,
            color: 'secondary',
            options: [
              {
                accessor: VM_TEMPLATE_ACTIONS.CHANGE_OWNER,
                name: T.ChangeOwner,
                disabled: true,
                isConfirmDialog: true,
                onSubmit: () => undefined,
              },
              {
                accessor: VM_TEMPLATE_ACTIONS.CHANGE_GROUP,
                name: T.ChangeGroup,
                disabled: true,
                isConfirmDialog: true,
                onSubmit: () => undefined,
              },
              {
                accessor: VM_TEMPLATE_ACTIONS.SHARE,
                disabled: true,
                name: T.Share,
                isConfirmDialog: true,
                onSubmit: () => undefined,
              },
              {
                accessor: VM_TEMPLATE_ACTIONS.UNSHARE,
                disabled: true,
                name: T.Unshare,
                isConfirmDialog: true,
                onSubmit: () => undefined,
              },
            ],
          },
          {
            tooltip: T.Lock,
            icon: Lock,
            selected: true,
            color: 'secondary',
            options: [
              {
                accessor: VM_TEMPLATE_ACTIONS.LOCK,
                name: T.Lock,
                isConfirmDialog: true,
                dialogProps: {
                  title: T.Lock,
                  children: MessageToConfirmAction,
                },
                onSubmit: (rows) => async () => {
                  const ids = rows?.map?.(({ original }) => original?.ID)
                  await Promise.all(ids.map((id) => lock({ id })))
                },
              },
              {
                accessor: VM_TEMPLATE_ACTIONS.UNLOCK,
                name: T.Unlock,
                isConfirmDialog: true,
                dialogProps: {
                  title: T.Unlock,
                  children: MessageToConfirmAction,
                },
                onSubmit: (rows) => async () => {
                  const ids = rows?.map?.(({ original }) => original?.ID)
                  await Promise.all(ids.map((id) => unlock(id)))
                },
              },
            ],
          },
          {
            accessor: VM_TEMPLATE_ACTIONS.DELETE,
            tooltip: T.Delete,
            icon: Trash,
            selected: true,
            color: 'error',
            options: [
              {
                isConfirmDialog: true,
                dialogProps: {
                  title: T.Delete,
                  children: MessageToConfirmAction,
                },
                onSubmit: (rows) => async () => {
                  const ids = rows?.map?.(({ original }) => original?.ID)
                  await Promise.all(ids.map((id) => remove({ id })))
                },
              },
            ],
          },
        ],
      }),
    [view]
  )
}

export default Actions
